@@ -8,31 +8,31 @@
 !-----------------------------------------------------------------------
 SUBROUTINE do_phonon(auxdyn)
   !-----------------------------------------------------------------------
-  !! This is the main driver of the phonon code. It assumes that the 
-  !! preparatory stuff has been already done.  
+  !! This is the main driver of the phonon code. It assumes that the
+  !! preparatory stuff has been already done.
   !! When the code calls this routine it has already read input
   !! decided which irreducible representations have to be calculated
   !! and it has set the variables that decide which work this routine
   !! will do. The parallel stuff has been already setup by the calling
   !! codes. This routine makes the two loops over
   !! the q-points and the irreps and does only the calculations
-  !! that have been decided by the driver routine.  
-  !! At a generic q-point, if necessary, it recalculates the band structure 
+  !! that have been decided by the driver routine.
+  !! At a generic q-point, if necessary, it recalculates the band structure
   !! calling pwscf again. Then it can calculate the response to an atomic
-  !! displacement, the dynamical matrix at that q-point, and the 
-  !! electron-phonon interaction at that q. At q=0 it can calculate 
+  !! displacement, the dynamical matrix at that q-point, and the
+  !! electron-phonon interaction at that q. At q=0 it can calculate
   !! the linear response to an electric field perturbation and hence the
   !! dielectric constant, the Born effective charges and the polarizability
-  !! at imaginary frequencies.  
+  !! at imaginary frequencies.
   !! At q=0, from the second order response to an electric field,
   !! it can calculate also the electro-optic and the raman tensors.
   !
 
   USE disp,            ONLY : nqs
   USE control_flags,   ONLY : use_gpu
-  USE control_ph,      ONLY : epsil, trans, qplot, only_init, &
-                              only_wfc, rec_code, where_rec, reduce_io
-  USE el_phon,         ONLY : elph, elph_mat, elph_simple, elph_epa
+  USE control_lr,      ONLY : rec_code, where_rec, reduce_io
+  USE control_ph,      ONLY : epsil, trans, qplot, only_init, only_wfc
+  USE el_phon,         ONLY : elph, elph_mat, elph_simple, elph_epa, elph_print
   !
   ! YAMBO >
   USE YAMBO,           ONLY : elph_yambo
@@ -48,6 +48,7 @@ SUBROUTINE do_phonon(auxdyn)
   USE buffers,        ONLY : close_buffer
   USE control_flags,  ONLY : use_gpu
   USE environment,   ONLY : print_cuda_info
+  USE control_lr,     ONLY : lmultipole
   
   IMPLICIT NONE
   !
@@ -86,7 +87,7 @@ SUBROUTINE do_phonon(auxdyn)
         CALL print_cuda_info(check_use_gpu=.true.) 
      ENDIF
      !
-     !  If only_wfc=.TRUE. the code computes only the wavefunctions 
+     !  If only_wfc=.TRUE. the code computes only the wavefunctions
      !
      IF (only_wfc) THEN
         where_rec='only_wfc'
@@ -103,7 +104,7 @@ SUBROUTINE do_phonon(auxdyn)
      !
      IF (epsil) CALL phescf()
      !
-     !  IF only_init is .true. the code computes only the 
+     !  IF only_init is .true. the code computes only the
      !  initialization parts.
      !
      IF (only_init) THEN
@@ -117,7 +118,8 @@ SUBROUTINE do_phonon(auxdyn)
      IF ( trans ) THEN
         !
         CALL phqscf()
-        CALL dynmatrix_new(iq)
+        IF (lmultipole) CALL write_drhoun()
+        IF (.NOT. lmultipole) CALL dynmatrix_new(iq)
         !
      END IF
      !
@@ -134,6 +136,7 @@ SUBROUTINE do_phonon(auxdyn)
               CALL ep_matrix_element_wannier()
            ELSE
               CALL elphon()
+              IF (.NOT. elph_ahc) CALL rediagonalize_dyn () 
            END IF
            !
         END IF
@@ -154,7 +157,9 @@ SUBROUTINE do_phonon(auxdyn)
            CALL elph_scdft()
         ELSEIF( elph_ahc ) THEN
            CALL elph_do_ahc()
-        ELSE 
+        ELSEIF( elph_print ) THEN
+           CALL elph_prt()
+        ELSE
            CALL elphsum()
         END IF
         !
