@@ -1,4 +1,4 @@
-# LaTeX2HTML 2021.2 (Released July 1, 2021)
+# LaTeX2HTML 2024 (Released January 1, 2024)
 # Associate labels original text with physical files.
 
 
@@ -13,9 +13,17 @@ $noresave{$key} = "$nosave";
 1;
 
 
-# LaTeX2HTML 2021.2 (Released July 1, 2021)
+# LaTeX2HTML 2024 (Released January 1, 2024)
 # labels from external_latex_labels array.
 
 
+$key = q/SubSec:badpara/;
+$external_latex_labels{$key} = q|4.4 Parallelization issues|; 
+$noresave{$key} = "$nosave";
+
+$key = q/pw2casino_info/;
+$external_latex_labels{$key} = q|3.5 Direct interface with <TT>CASINO</TT>|; 
+$noresave{$key} = "$nosave";
+
 1;
 
