@@ -20,8 +20,8 @@ MODULE Coul_cut_2D
   !! 2D Coulomb cutoff.
   !
   USE kinds,       ONLY : DP
-  USE constants,   ONLY : tpi, pi
   !
+  IMPLICIT NONE
   SAVE
   !
   LOGICAL :: do_cutoff_2D = .FALSE.
@@ -43,12 +43,10 @@ SUBROUTINE cutoff_fact()
   !! a vector called \(\text{cutoff_2D}(:)\), to be re-used in various routines.  
   !! See Eq.(24) of PRB 96, 075448
   !
-  USE kinds
   USE io_global,    ONLY : stdout
   USE gvect,        ONLY : g, ngm, ngmx
   USE cell_base,    ONLY : alat, celldm, at
-  !
-  IMPLICIT NONE
+  USE constants,    ONLY : tpi
   !
   ! ... local variables
   !
@@ -57,6 +55,7 @@ SUBROUTINE cutoff_fact()
   REAL(DP) :: Gzlz, Gplz
   !
   ALLOCATE( cutoff_2D(ngmx) ) 
+  !$acc enter data create(cutoff_2D)
   !
   ! Message to indicate that the cutoff is active. 
   WRITE(stdout, *) "----2D----2D----2D----2D----2D----2D----2D----2D----2D----2D----2D----2D"
@@ -73,13 +72,14 @@ SUBROUTINE cutoff_fact()
   DO i = 1, 2
      IF (ABS(at(3,i))>1d-8) WRITE(stdout, *) "2D CODE WILL NOT WORK, 2D MATERIAL NOT IN X-Y PLANE!!"
   ENDDO
-  ! define cutoff distnce and compute cutoff factor
+  ! define cutoff distance and compute cutoff factor
   lz = 0.5d0*at(3,3)*alat
   DO ng = 1, ngm
      Gplz = SQRT( g(1,ng)**2 + g(2,ng)**2 )*tpi*lz/alat
      Gzlz = g(3,ng)*tpi*lz/alat
      cutoff_2D(ng) = 1.0d0 - EXP(-Gplz)*COS(Gzlz)
   ENDDO
+  !$acc update device(cutoff_2D)
   !
   RETURN
   !
@@ -92,7 +92,6 @@ SUBROUTINE cutoff_lr_Vloc( )
   !! 2D calculations.  
   !! See Eq. (32) of PRB 96, 075448.
   !
-  USE kinds
   USE constants,    ONLY : fpi, e2, eps8
   USE fft_base,     ONLY : dfftp
   USE gvect,        ONLY : ngm, gg, g, ngmx
@@ -101,8 +100,6 @@ SUBROUTINE cutoff_lr_Vloc( )
   USE uspp_param,   ONLY : upf
   USE cell_base,    ONLY : omega, tpiba2
   !
-  IMPLICIT NONE
-  !
   ! ... local variables
   !
   INTEGER :: ng, nt, ng0 
@@ -139,13 +136,10 @@ SUBROUTINE cutoff_local( aux )
   !! routine \(\texttt{cutoff_lr_Vloc}\).  
   !! See Eq. (33) of PRB 96, 075448
   !
-  USE kinds
   USE gvect,      ONLY : ngm
   USE vlocal,     ONLY : strf
   USE ions_base,  ONLY : nsp
   !
-  IMPLICIT NONE
-  !
   COMPLEX(DP), INTENT(INOUT):: aux(ngm)
   !! input: local part of ionic potential 
   !
@@ -161,7 +155,6 @@ SUBROUTINE cutoff_local( aux )
   !
 END SUBROUTINE cutoff_local
 !
-!
 !----------------------------------------------------------------------
 SUBROUTINE cutoff_hartree( rhog, aux1, ehart )
   !----------------------------------------------------------------------
@@ -169,11 +162,8 @@ SUBROUTINE cutoff_hartree( rhog, aux1, ehart )
   !! energy accordingly in G-space.  
   !! See Eq. (34) and (41) of PRB 96, 075448
   !
-  USE kinds
   USE gvect,       ONLY : ngm, gg , gstart
   USE io_global,   ONLY : stdout
-  !
-  IMPLICIT NONE
   ! 
   COMPLEX(DP), INTENT(IN) :: rhog(ngm)
   !! local potential
@@ -188,6 +178,9 @@ SUBROUTINE cutoff_hartree( rhog, aux1, ehart )
   REAL(DP) :: fac
   REAL(DP) :: rgtot_re, rgtot_im
   !  
+  !$acc data copyin(rhog) copy(aux1)
+  !
+  !$acc parallel loop reduction(+:ehart)
   DO ig = gstart, ngm
      !
      fac = 1.D0 / gg(ig) * cutoff_2D(ig)
@@ -202,33 +195,34 @@ SUBROUTINE cutoff_hartree( rhog, aux1, ehart )
      !
   ENDDO
   !
+  !$acc end data
+  !
   RETURN
   !
 END SUBROUTINE cutoff_hartree
 !
-!
 !----------------------------------------------------------------------
-SUBROUTINE cutoff_ewald( alpha, ewaldg, omega )
+FUNCTION cutoff_ewald( gamma_only, alpha, omega ) RESULT (ewaldg)
   !----------------------------------------------------------------------
   !! This subroutine defines computes the cutoff version of the 
   !! Ewald sum in G space.
   !! See Eq. (46) of PRB 96, 075448
   !
-  USE kinds
   USE gvect,      ONLY : ngm, gg, gstart
   USE ions_base,  ONLY : zv, nsp, nat, ityp
   USE cell_base,  ONLY : tpiba2, alat
   USE vlocal,     ONLY : strf
   USE io_global,  ONLY : stdout
+  USE constants,  ONLY : tpi, fpi
   !
-  IMPLICIT NONE
-  !
+  LOGICAL, INTENT(IN) :: gamma_only
+  !! If true, use only half of the Fourier components ("Gamma tricks")
   REAL(DP), INTENT(IN) :: alpha
   !! tuning parameter for Ewald LR/SR separation
-  REAL(DP), INTENT(INOUT) :: ewaldg
-  !! Ewald sum
   REAL(DP), INTENT(IN) :: omega
   !! unit-cell volume
+  REAL(DP) :: ewaldg
+  !! Ewald sum
   !
   ! ... local variables
   !
@@ -254,7 +248,8 @@ SUBROUTINE cutoff_ewald( alpha, ewaldg, omega )
      ewaldg = ewaldg +  ABS(rhon)**2 * EXP( - gg(ng) * tpiba2 /&
               alpha / 4.d0) / gg(ng)*cutoff_2D(ng) / tpiba2
   ENDDO
-  ewaldg = 2.d0 * tpi / omega * ewaldg
+  ewaldg = fpi / omega * ewaldg
+  IF ( gamma_only ) ewaldg = 2.0_dp*ewaldg
   !
   ! ... here add the other constant term (Phi_self)
   !
@@ -267,8 +262,7 @@ SUBROUTINE cutoff_ewald( alpha, ewaldg, omega )
   !  
   RETURN
   !
-END SUBROUTINE cutoff_ewald
-!
+END FUNCTION cutoff_ewald
 !
 !----------------------------------------------------------------------
 SUBROUTINE cutoff_force_ew( aux, alpha )
@@ -280,12 +274,9 @@ SUBROUTINE cutoff_force_ew( aux, alpha )
   !! looks somewhat different from what is implemented in the code, but it is
   !! equivalent).
   !
-  USE kinds
   USE gvect,        ONLY : ngm, gg , gstart
   USE cell_base,    ONLY : tpiba2, alat
   !
-  IMPLICIT NONE
-  !
   COMPLEX(DP), INTENT(INOUT) :: aux(ngm)
   !! long-range part of the ionic potential
   REAL(DP), INTENT(IN) :: alpha
@@ -304,16 +295,14 @@ SUBROUTINE cutoff_force_ew( aux, alpha )
   !
 END SUBROUTINE cutoff_force_ew
 !
-!
 !----------------------------------------------------------------------
-SUBROUTINE cutoff_force_lc( aux, forcelc )
+SUBROUTINE cutoff_force_lc( gamma_only, aux, forcelc )
   !----------------------------------------------------------------------
   !! This subroutine re-adds the cutoff contribution from the long-range 
   !! local part of the ionic potential to the forces. In the 2D code, this 
   !! contribution is missing from the \(\text{Vloc}\).  
   !! See Eq. (54) of PRB 96, 075448.
   !
-  USE kinds
   USE gvect,         ONLY : ngm, gg, g , gstart
   USE constants,     ONLY : fpi, e2, eps8, tpi
   USE uspp_param,    ONLY : upf 
@@ -322,8 +311,8 @@ SUBROUTINE cutoff_force_lc( aux, forcelc )
   USE io_global,     ONLY : stdout
   USE fft_base,      ONLY : dfftp
   !
-  IMPLICIT NONE
-  !
+  LOGICAL, INTENT(IN) :: gamma_only
+  !! If true, use only half of the Fourier components ("Gamma tricks")
   COMPLEX(DP), INTENT(IN) :: aux(dfftp%nnr)
   !! local ionic potential
   REAL(DP), INTENT(INOUT) :: forcelc(3,nat)
@@ -331,15 +320,20 @@ SUBROUTINE cutoff_force_lc( aux, forcelc )
   !
   ! ... local variables
   !
-  REAL(DP) :: arg
+  REAL(DP) :: arg, fac
   INTEGER :: ig, na, ipol
   !
+  IF ( gamma_only ) THEN
+     fac = 2.0_dp
+  ELSE
+     fac = 1.0_dp
+  END IF
   DO na = 1, nat
      DO ig = gstart, ngm 
         arg = (g(1,ig) * tau(1,na) + g(2,ig) * tau(2,na) + &
                g(3,ig) * tau(3,na) ) * tpi
         DO ipol = 1, 3
-           forcelc(ipol,na) = forcelc (ipol,na) + tpi / alat * &
+           forcelc(ipol,na) = forcelc (ipol,na) + fac * tpi / alat * &
                  g(ipol,ig) * lr_Vloc(ig, ityp(na)) * omega  * &
                 ( SIN(arg)*DBLE(aux(ig)) + COS(arg)*AIMAG(aux(ig)) )
         ENDDO
@@ -350,9 +344,8 @@ SUBROUTINE cutoff_force_lc( aux, forcelc )
   !
 END SUBROUTINE cutoff_force_lc
 !
-!
 !----------------------------------------------------------------------
-SUBROUTINE cutoff_stres_evloc( rho_G, strf, evloc )
+SUBROUTINE cutoff_stres_evloc( gamma_only, rho_G, strf, evloc )
   !----------------------------------------------------------------------
   !! This subroutine adds the contribution from the cutoff long-range part
   !! of the local part of the ionic potential to \(\text{evloc}\).  
@@ -362,14 +355,13 @@ SUBROUTINE cutoff_stres_evloc( rho_G, strf, evloc )
   !! Indeed, it is "hidden" in the sum of KS eigenvalues. That is why we need 
   !! to re-compute it here for the stress.
   !
-  USE kinds
   USE ions_base,  ONLY: ntyp => nsp
   USE gvect,      ONLY: ngm, gstart
   USE io_global,  ONLY: stdout
   USE fft_base,   ONLY: dfftp
   !
-  IMPLICIT NONE
-  !
+  LOGICAL, INTENT(IN) :: gamma_only
+  !! If true, use only half of the Fourier components ("Gamma tricks")
   COMPLEX(DP), INTENT(IN) :: rho_G(dfftp%nnr)
   !! charge density in G space
   COMPLEX(DP), INTENT(IN) :: strf(ngm,ntyp)
@@ -380,6 +372,13 @@ SUBROUTINE cutoff_stres_evloc( rho_G, strf, evloc )
   ! ... local variables
   !
   INTEGER :: ng, nt
+  REAL(DP) :: fac
+  !
+  IF ( gamma_only ) THEN
+     fac = 2.0_dp
+  ELSE
+     fac = 1.0_dp
+  END IF
   !
   !$acc data present_or_copyin(rho_G,strf)
   !
@@ -390,7 +389,7 @@ SUBROUTINE cutoff_stres_evloc( rho_G, strf, evloc )
   DO nt = 1, ntyp
     DO ng = gstart, ngm
        evloc = evloc + DBLE( CONJG(rho_G(ng)) * strf(ng,nt) ) &
-                       * lr_Vloc(ng,nt)
+                       * lr_Vloc(ng,nt) * fac
     ENDDO
   ENDDO
   !
@@ -400,15 +399,13 @@ SUBROUTINE cutoff_stres_evloc( rho_G, strf, evloc )
   !
 END SUBROUTINE cutoff_stres_evloc
 !
-!
 !----------------------------------------------------------------------
-SUBROUTINE cutoff_stres_sigmaloc( rho_G, strf, sigmaloc )
+SUBROUTINE cutoff_stres_sigmaloc( gamma_only, rho_G, strf, sigmaloc )
   !----------------------------------------------------------------------
   !! This subroutine adds the contribution from the cutoff long-range part 
   !! of the local part of the ionic potential to the rest of the 
   !! \(\text{sigmaloc}\). That is, the rest of Eq. (63) of PRB 96, 075448.
   !
-  USE kinds
   USE ions_base,   ONLY : ntyp => nsp
   USE constants,   ONLY : eps8
   USE gvect,       ONLY : ngm, gstart, g, gg
@@ -416,8 +413,8 @@ SUBROUTINE cutoff_stres_sigmaloc( rho_G, strf, sigmaloc )
   USE io_global,   ONLY : stdout
   USE fft_base,    ONLY : dfftp
   !
-  IMPLICIT NONE
-  !
+  LOGICAL, INTENT(IN) :: gamma_only
+  !! If true, use only half of the Fourier components ("Gamma tricks")
   COMPLEX(DP), INTENT(IN) :: rho_G(dfftp%nnr)
   !! charge density in G space
   COMPLEX(DP), INTENT(IN) :: strf(ngm,ntyp)
@@ -428,9 +425,15 @@ SUBROUTINE cutoff_stres_sigmaloc( rho_G, strf, sigmaloc )
   !
   INTEGER  :: ng, nt, l, m
   REAL(DP) :: Gp, G2lzo2Gp, beta, dlr_Vloc1, dlr_Vloc2, dlr_Vloc3, &
-              no_lm_dep
+              no_lm_dep, fac
   REAL(DP) :: sigmaloc11, sigmaloc31, sigmaloc21, sigmaloc32, &
               sigmaloc22, sigmaloc33
+  !
+  IF ( gamma_only ) THEN
+     fac = 2.0_dp
+  ELSE
+     fac = 1.0_dp
+  END IF
   ! 
   !$acc data present_or_copyin(rho_G,strf)
   !
@@ -464,7 +467,7 @@ SUBROUTINE cutoff_stres_sigmaloc( rho_G, strf, sigmaloc )
                                * (1._DP- beta + gg(ng)*tpiba2/4._DP)
         dlr_Vloc3 = -1._DP / (gg(ng)*tpiba2) * lr_Vloc(ng,nt) &
                                * (1._DP+ gg(ng)*tpiba2/4._DP)
-        no_lm_dep = DBLE( CONJG( rho_G(ng) ) &
+        no_lm_dep = fac * DBLE( CONJG( rho_G(ng) ) &
                           * strf(ng,nt) ) * 2._DP * tpiba2
         sigmaloc11 = sigmaloc11 + no_lm_dep * dlr_Vloc1 * g(1,ng) * g(1,ng)
         sigmaloc21 = sigmaloc21 + no_lm_dep * dlr_Vloc2 * g(2,ng) * g(1,ng)
@@ -489,14 +492,12 @@ SUBROUTINE cutoff_stres_sigmaloc( rho_G, strf, sigmaloc )
   !
 END SUBROUTINE cutoff_stres_sigmaloc
 !
-!
 !----------------------------------------------------------------------
 SUBROUTINE cutoff_stres_sigmahar( rho_G, sigmahar )
   !----------------------------------------------------------------------
   !! This subroutine cuts off the Hartree part of the stress.  
   !! See Eq. (62) of PRB 96, 075448.
   !
-  USE kinds
   USE gvect,      ONLY: ngm, gstart
   USE constants,  ONLY: eps8
   USE cell_base,  ONLY: tpiba2, alat, tpiba
@@ -504,8 +505,6 @@ SUBROUTINE cutoff_stres_sigmahar( rho_G, sigmahar )
   USE fft_base,   ONLY: dfftp
   USE gvect,      ONLY: g, gg
   !
-  IMPLICIT NONE
-  !
   COMPLEX(DP), INTENT(IN) :: rho_G(dfftp%nnr)
   !! charge density in G-space
   REAL(DP), INTENT(INOUT) :: sigmahar(3,3)
@@ -572,22 +571,20 @@ SUBROUTINE cutoff_stres_sigmahar( rho_G, sigmahar )
   !
 END SUBROUTINE cutoff_stres_sigmahar
 !
-!
 !----------------------------------------------------------------------
-SUBROUTINE cutoff_stres_sigmaewa( alpha, sdewald, sigmaewa )
+SUBROUTINE cutoff_stres_sigmaewa( gamma_only, alpha, sdewald, sigmaewa )
   !----------------------------------------------------------------------
   !! This subroutine cuts off the Ewald part of the stress.  
   !! See Eq. (64) in PRB 96 075448
   !
-  USE kinds
   USE ions_base,   ONLY : nat, zv, tau, ityp
-  USE constants,   ONLY : e2, eps8
+  USE constants,   ONLY : tpi, e2, eps8
   USE gvect,       ONLY : ngm, gstart, g, gg
   USE cell_base,   ONLY : tpiba2, alat, omega, tpiba
   USE io_global,   ONLY : stdout
   !
-  IMPLICIT NONE
-  !
+  LOGICAL, INTENT(IN) :: gamma_only
+  !! If true, use only half of the Fourier components ("Gamma tricks")
   REAL(DP), INTENT(IN) :: alpha
   !! tuning param for LR/SR separation
   REAL(DP), INTENT(INOUT) :: sigmaewa(3,3)
@@ -602,11 +599,15 @@ SUBROUTINE cutoff_stres_sigmaewa( alpha, sdewald, sigmaewa )
   REAL(DP) :: sigma11, sigma21, sigma22, sigma31, sigma32, sigma33
   COMPLEX(DP) :: rhostar
   !
+  IF ( gamma_only ) THEN
+     fact = 2.0_dp
+  ELSE
+     fact = 1.0_dp
+  END IF
   ntyp = SIZE(zv)
   !
   ! ... g(1) is a problem if it's G=0, because we divide by G^2. 
   !     So start at gstart.
-  !     fact=1.0d0, gamma_only not implemented
   !     G=0 componenent of the long-range part of the local part of the 
   !     pseudopotminus the Hartree potential is set to 0.
   !     in other words, sdewald=0.  
@@ -639,7 +640,7 @@ SUBROUTINE cutoff_stres_sigmaewa( alpha, sdewald, sigmaewa )
         rhostar = rhostar + CMPLX(zv(ityp(na)),KIND=DP) * CMPLX(COS(arg),SIN(arg),KIND=DP)
      ENDDO
      rhostar = rhostar / CMPLX(omega,KIND=DP)
-     sewald = tpi * e2 * EXP(-g2a) / g2* cutoff_2D(ng) * ABS(rhostar)**2
+     sewald = fact * tpi * e2 * EXP(-g2a) / g2* cutoff_2D(ng) * ABS(rhostar)**2
      ! ... sewald is an other diagonal term that is similar to the diagonal terms 
      !     in the other stress contributions. It basically gives a term prop to 
      !     the ewald energy
@@ -671,5 +672,4 @@ SUBROUTINE cutoff_stres_sigmaewa( alpha, sdewald, sigmaewa )
   !
 END SUBROUTINE cutoff_stres_sigmaewa
 !
-!
 END MODULE Coul_cut_2D
