@@ -46,7 +46,7 @@ SUBROUTINE addusdens_g(rho)
   USE gvect,                ONLY : ngm, gg, g, &
                                    eigts1, eigts2, eigts3, mill
   USE noncollin_module,     ONLY : noncolin, nspin_mag
-  USE uspp,                 ONLY : okvan, becsum, becsum_d
+  USE uspp,                 ONLY : okvan, becsum
   USE uspp_param,           ONLY : upf, lmaxq, nh, nhm
   USE control_flags,        ONLY : gamma_only
   USE mp_pools,             ONLY : inter_pool_comm
@@ -136,14 +136,10 @@ SUBROUTINE addusdens_g(rho)
               nb = nb + 1
               !tbecsum(:,nb,:) = becsum(1:nij,na,1:nspin_mag)
               !
-              !$acc parallel loop collapse(2)
+              !$acc parallel loop collapse(2) present(becsum)
               DO im = 1, nspin_mag
                  DO ij = 1, nij
-#if defined(__CUDA)
-                   tbecsum(ij,nb,im) = becsum_d(ij,na,im)
-#else
                    tbecsum(ij,nb,im) = becsum(ij,na,im)
-#endif
                  ENDDO
               ENDDO
               !
