@@ -1,13 +1,10 @@
 
-! Copyright (C) 2001-2012 Quantum ESPRESSO group
+! Copyright (C) 2001-2025 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
 ! or http://www.gnu.org/copyleft/gpl.txt .
 !
-! TB
-! included deallocation of forcefield of gate 'forcegate'
-!
 !----------------------------------------------------------------------
 SUBROUTINE clean_pw( lflag )
   !----------------------------------------------------------------------
@@ -22,7 +19,6 @@ SUBROUTINE clean_pw( lflag )
   !! phonon, vc-relax). Beware: the new calculation should not CALL any
   !! of the routines mentioned above.
   !
-  USE basis,                ONLY : swfcatom
   USE cellmd,               ONLY : lmovecell
   USE ions_base,            ONLY : deallocate_ions_base
   USE fixed_occ,            ONLY : f_inp
@@ -36,12 +32,12 @@ SUBROUTINE clean_pw( lflag )
                                    vrs, kedtau, destroy_scf_type, vnew
   USE symm_base,            ONLY : irt
   USE symme,                ONLY : sym_rho_deallocate
-  USE wavefunctions,        ONLY : evc, psic, psic_nc
+  USE wavefunctions,        ONLY : deallocate_wfc, psic, psic_nc
   USE uspp,                 ONLY : deallocate_uspp
-  USE uspp_data,            ONLY : deallocate_uspp_data
   USE uspp_param,           ONLY : upf
+  USE atwfc_mod,            ONLY : deallocate_tab_atwfc
   USE m_gth,                ONLY : deallocate_gth
-  USE ldaU,                 ONLY : deallocate_hubbard
+  USE ldaU,                 ONLY : deallocate_hubbard, order_um
   USE extfield,             ONLY : forcefield, forcegate
   USE fft_base,             ONLY : dfftp, dffts  
   USE fft_base,             ONLY : pstickdealloc
@@ -62,15 +58,11 @@ SUBROUTINE clean_pw( lflag )
   USE exx,                  ONLY : deallocate_exx
   USE Coul_cut_2D,          ONLY : cutoff_2D, lr_Vloc 
   !
-  USE control_flags,        ONLY : ts_vdw, mbd_vdw
+  USE control_flags,        ONLY : ts_vdw, mbd_vdw, use_gpu
   USE tsvdw_module,         ONLY : tsvdw_finalize
   USE libmbd_interface,     ONLY : clean_mbd
   USE dftd3_qe,             ONLY : dftd3_clean
   !
-  USE wavefunctions_gpum,   ONLY : deallocate_wavefunctions_gpu
-  USE wvfct_gpum,           ONLY : deallocate_wvfct_gpu
-  USE scf_gpum,             ONLY : deallocate_scf_gpu
-  !
   USE control_flags,        ONLY : sic, scissor
   USE sic_mod,              ONLY : deallocate_sic
   USE sci_mod,              ONLY : deallocate_scissor
@@ -79,6 +71,10 @@ SUBROUTINE clean_pw( lflag )
 #if defined (__ENVIRON)
   USE plugin_flags,         ONLY : use_environ
   USE environ_base_module,  ONLY : clean_environ
+#endif
+#if defined (__OSCDFT)
+   USE plugin_flags,     ONLY : use_oscdft
+   USE oscdft_base,      ONLY : oscdft_ctx
 #endif
   !
   IMPLICIT NONE
@@ -88,7 +84,7 @@ SUBROUTINE clean_pw( lflag )
   !
   ! ... local variables
   !
-  INTEGER :: nt, nr1, nr2, nr3
+  INTEGER :: nt, nr1, nr2, nr3, istat
   !
   IF ( lflag ) THEN
      !
@@ -138,25 +134,27 @@ SUBROUTINE clean_pw( lflag )
   CALL destroy_scf_type( v    )
   CALL destroy_scf_type( vnew )
   !
+  IF ( ALLOCATED( order_um))     DEALLOCATE (order_um) 
   IF ( ALLOCATED( kedtau ) )     DEALLOCATE( kedtau )
   IF ( ALLOCATED( vltot  ) )     DEALLOCATE( vltot  )
   IF ( ALLOCATED( rho_core  ) )  DEALLOCATE( rho_core  )
   IF ( ALLOCATED( rhog_core ) )  DEALLOCATE( rhog_core )
   IF ( ALLOCATED( psic    ) )    DEALLOCATE( psic    )
   IF ( ALLOCATED( psic_nc ) )    DEALLOCATE( psic_nc )
+  !$acc exit data delete(vrs)
   IF ( ALLOCATED( vrs     ) )    DEALLOCATE( vrs     )
-  CALL deallocate_scf_gpu()
   !
   ! ... arrays allocated in allocate_locpot.f90 ( and never deallocated )
   !
   IF ( ALLOCATED( vloc )      )  DEALLOCATE( vloc      )
-  IF ( ALLOCATED( cutoff_2D ) )  DEALLOCATE( cutoff_2D )
+  IF ( ALLOCATED( cutoff_2D ) ) THEN
+    !$acc exit data delete(cutoff_2d)
+    DEALLOCATE( cutoff_2D )
+  ENDIF
   IF ( ALLOCATED( lr_Vloc )   )  DEALLOCATE( lr_Vloc   )
   IF ( ALLOCATED( strf )      )  DEALLOCATE( strf      )
   !
-  ! ... arrays allocated in allocate_nlpot.f90 ( and never deallocated )
-  !
-  CALL deallocate_uspp_data()
+  CALL deallocate_tab_atwfc()
   CALL deallocate_uspp() 
   !
   CALL deallocate_gth( lflag ) 
@@ -167,17 +165,14 @@ SUBROUTINE clean_pw( lflag )
   !
   !$acc exit data delete(g2kin)
   IF ( ALLOCATED( g2kin ) )      DEALLOCATE( g2kin )
-  CALL deallocate_wvfct_gpu()
+  !$acc exit data delete(et)
   IF ( ALLOCATED( et ) )         DEALLOCATE( et )
   IF ( ALLOCATED( wg ) )         DEALLOCATE( wg )
   IF ( ALLOCATED( btype ) )      DEALLOCATE( btype )
   !
   ! ... arrays allocated in allocate_wfc.f90 ( and never deallocated )
   !
-  IF ( ALLOCATED( evc ) )        DEALLOCATE( evc )
-  IF ( ALLOCATED( swfcatom ) )   DEALLOCATE( swfcatom )
-  !
-  CALL deallocate_wavefunctions_gpu()
+  CALL deallocate_wfc ( )
   !
   ! ... fft structures allocated in data_structure.f90  
   !
@@ -194,6 +189,7 @@ SUBROUTINE clean_pw( lflag )
   !
   nr1 = dffts%nr1; nr2 = dffts%nr2; nr3 = dffts%nr3
   CALL fft_type_deallocate( dffts )
+  !
   dffts%nr1 = nr1; dffts%nr2 = nr2; dffts%nr3 = nr3
   !
   ! ... stick-owner matrix allocated in sticks_base
@@ -232,6 +228,11 @@ SUBROUTINE clean_pw( lflag )
 #endif 
 #if defined (__ENVIRON)
   IF (use_environ) CALL clean_environ('PW', lflag)
+#endif
+#if defined (__OSCDFT)
+     IF (use_oscdft .AND. (oscdft_ctx%inp%oscdft_type==2)) THEN
+        DEALLOCATE (oscdft_ctx%inp%occupation)
+     ENDIF
 #endif
   CALL   plugin_clean('PW', lflag) 
   !
