@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2001-2020 Quantum ESPRESSO group
+! Copyright (C) 2001-2023 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -25,7 +25,7 @@ MODULE io_rho_xml
     SUBROUTINE write_scf ( rho, nspin )
       !
       USE paw_variables,    ONLY : okpaw
-      USE ldaU,             ONLY : lda_plus_u, hub_back, lda_plus_u_kind, nsg
+      USE ldaU,             ONLY : lda_plus_u, hub_back, lda_plus_u_kind, nsg, apply_U, order_um
       USE two_chem,         ONLY : twochem
       USE xc_lib,           ONLY : xclib_dft_is
       USE noncollin_module, ONLY : noncolin, domag
@@ -72,7 +72,6 @@ MODULE io_rho_xml
               root_bgrp, intra_bgrp_comm, &
               bg(:,1)*tpiba, bg(:,2)*tpiba, bg(:,3)*tpiba, &
               gamma_only, mill, ig_l2g, rho%kin_g(:,1:nspin_) )
-         WRITE(stdout,'(5x,"Writing meta-gga kinetic term")')
       ENDIF
 
       ! Then write the other terms to separate files
@@ -83,7 +82,11 @@ MODULE io_rho_xml
             OPEN ( NEWUNIT=iunocc, FILE = TRIM(dirname) // 'occup.txt', &
                  FORM='formatted', STATUS='unknown' )
             IF (lda_plus_u_kind.EQ.0) THEN
-               WRITE( iunocc, * , iostat = ierr) rho%ns
+               IF (noncolin) THEN
+                   WRITE( iunocc, * , iostat = ierr) rho%ns_nc
+               ELSE   
+                  WRITE( iunocc, * , iostat = ierr) rho%ns
+               ENDIF
                IF (hub_back) WRITE( iunocc, * , iostat = ierr) rho%nsb
             ELSEIF (lda_plus_u_kind.EQ.1) THEN
                IF (noncolin) THEN
@@ -100,6 +103,18 @@ MODULE io_rho_xml
          IF ( ionode ) THEN
             CLOSE( UNIT = iunocc, STATUS = 'KEEP' )
          ENDIF
+         IF (apply_U) THEN 
+           IF (ionode) THEN 
+             OPEN (NEWUNIT=iunocc, FILE= TRIM(dirname) // 'Hubbard_m_order.txt', FORM='formatted', STATUS='unknown')
+             WRITE (iunocc, *, iostat = ierr ) SHAPE(order_um) 
+             WRITE (iunocc, '(14I7)', iostat = ierr)  order_um 
+           END IF 
+           CALL mp_bcast( ierr, ionode_id, intra_image_comm )
+           IF ( ierr/=0 ) CALL errore('write_scf', 'Writing ldaU Hubbard m order', 1)
+           IF ( ionode ) THEN
+              CLOSE( UNIT = iunocc, STATUS = 'KEEP' )
+           ENDIF
+         END IF 
          !
       END IF
       !
@@ -124,11 +139,14 @@ MODULE io_rho_xml
     SUBROUTINE read_scf ( rho, nspin, gamma_only )
       !
       USE scf,              ONLY : scf_type
+      USE control_flags,    ONLY : lscf
       USE paw_variables,    ONLY : okpaw
       USE ldaU,             ONLY : lda_plus_u, starting_ns, hub_back, &
-                                   lda_plus_u_kind, nsg
+                                   lda_plus_u_kind, nsg, apply_U, order_um
+      use lsda_mod,         ONLY : magtot
       USE noncollin_module, ONLY : noncolin, domag
-      USE gvect,            ONLY : ig_l2g
+      USE cell_base,        ONLY : omega
+      USE gvect,            ONLY : ig_l2g, gstart
       USE xc_lib,           ONLY : xclib_dft_is
       USE io_files,         ONLY : restart_dir
       USE io_global,        ONLY : ionode, ionode_id, stdout
@@ -144,6 +162,7 @@ MODULE io_rho_xml
       CHARACTER(LEN=256) :: dirname
       LOGICAL :: lexist
       INTEGER :: nspin_, iunocc, iunpaw, ierr
+      INTEGER :: ll, nat
       INTEGER, EXTERNAL :: find_free_unit
 
       dirname = restart_dir ( )
@@ -159,6 +178,13 @@ MODULE io_rho_xml
            ig_l2g, nspin_, rho%of_g, gamma_only )
       IF ( nspin > nspin_) rho%of_g(:,nspin_+1:nspin) = (0.0_dp, 0.0_dp)
       !
+      ! update total magnetization
+      IF ( nspin == 2 ) THEN
+         magtot = 0.0_dp
+         IF ( gstart == 2 ) magtot = rho%of_g(1, 2) * omega
+         CALL mp_sum(magtot, intra_image_comm)
+      END IF
+      !
       ! read kinetic energy density
       IF ( xclib_dft_is('meta') ) THEN
          CALL read_rhog( TRIM(dirname) // "ekin-density", &
@@ -182,7 +208,11 @@ MODULE io_rho_xml
             OPEN ( NEWUNIT=iunocc, FILE = TRIM(dirname) // 'occup.txt', &
                  FORM='formatted', STATUS='old', IOSTAT=ierr )
             IF (lda_plus_u_kind.EQ.0) THEN
-               READ( UNIT = iunocc, FMT = *, iostat = ierr ) rho%ns
+               IF (noncolin) THEN
+                  READ( UNIT = iunocc, FMT = *, iostat = ierr ) rho%ns_nc
+               ELSE        
+                  READ( UNIT = iunocc, FMT = *, iostat = ierr ) rho%ns
+               ENDIF   
                IF (hub_back) READ( UNIT = iunocc, FMT = * , iostat = ierr) rho%nsb
             ELSEIF (lda_plus_u_kind.EQ.1) THEN
                IF (noncolin) THEN
@@ -202,7 +232,11 @@ MODULE io_rho_xml
             CLOSE( UNIT = iunocc, STATUS = 'KEEP')
          ELSE
             IF (lda_plus_u_kind.EQ.0) THEN
-               rho%ns(:,:,:,:) = 0.D0
+               IF (noncolin) THEN
+                  rho%ns_nc(:,:,:,:) = 0.D0      
+               ELSE        
+                  rho%ns(:,:,:,:) = 0.D0
+               ENDIF
                IF (hub_back) rho%nsb(:,:,:,:) = 0.D0
             ELSEIF (lda_plus_u_kind.EQ.1) THEN
                IF (noncolin) THEN
@@ -216,7 +250,11 @@ MODULE io_rho_xml
          ENDIF
          !
          IF (lda_plus_u_kind.EQ.0) THEN
-            CALL mp_sum(rho%ns, intra_image_comm) 
+            IF (noncolin) THEN
+                CALL mp_sum(rho%ns_nc, intra_image_comm)
+            ELSE    
+                CALL mp_sum(rho%ns, intra_image_comm)
+            ENDIF    
             IF (hub_back) CALL mp_sum(rho%nsb, intra_image_comm)   
          ELSEIF (lda_plus_u_kind.EQ.1) THEN
             IF (noncolin) THEN
@@ -231,8 +269,38 @@ MODULE io_rho_xml
          ! If projections on Hubbard manifold are read from file, there is no
          ! need to set starting values: reset them to prevent any problem
          starting_ns = -1.0_dp
-         !
-      ENDIF
+         IF (apply_U) THEN
+           IF (ionode) THEN 
+             OPEN ( NEWUNIT=iunocc, FILE = TRIM(dirname) // 'Hubbard_m_order.txt', &
+                 FORM='formatted', STATUS='old', IOSTAT=ierr )
+             READ(iunocc,fmt=*,iostat=ierr) ll, nspin_, nat 
+           END IF 
+           CALL mp_bcast( ierr, ionode_id, intra_image_comm )
+           IF ( ierr/=0 ) THEN 
+              IF (lscf) THEN 
+                CALL infomsg('read_scf',&
+                   'OR-DFT+U from saved potential: m_order not found, I will redo U calc first' )
+                apply_U = .FALSE. 
+              ELSE 
+                call errore('read_scf', 'OR-DFT+U nscf calculation, but m_order not found ... stopping',1) 
+              END IF 
+           ELSE 
+              CALL mp_bcast(ll, ionode_id, intra_image_comm)
+              CALL mp_bcast(nspin_, ionode_id, intra_image_comm) 
+              CALL mp_bcast(nat, ionode_id, intra_image_comm)  
+              IF (ALLOCATED(order_um)) DEALLOCATE (order_um) 
+              ALLOCATE (order_um(ll, nspin_, nat)) 
+              order_um = 0 
+              IF (ionode) READ(iunocc,*) order_um 
+              IF (ionode) CLOSE(iunocc, STATUS='KEEP') 
+              CALL mp_bcast( ierr, ionode_id, intra_image_comm )
+              IF ( ierr/=0 ) CALL errore('read_scf', 'Reading ldaU ns', 1)
+              !
+              CALL mp_sum(order_um, intra_image_comm) 
+           END IF 
+           !
+         ENDIF
+       END IF   
       !
       IF ( okpaw ) THEN
          !
