@@ -1,5 +1,5 @@
 !
-! Copyright (C) 2001-2020 Quantum ESPRESSO group
+! Copyright (C) 2001-2025 Quantum ESPRESSO group
 ! This file is distributed under the terms of the
 ! GNU General Public License. See the file `License'
 ! in the root directory of the present distribution,
@@ -20,7 +20,7 @@ SUBROUTINE v_of_rho( rho, rho_core, rhog_core, &
   USE noncollin_module, ONLY : noncolin, nspin_lsda
   USE ions_base,        ONLY : nat, tau
   USE ldaU,             ONLY : lda_plus_u, lda_plus_u_kind, ldmx_b, &
-                               nsg, v_nsg 
+                               nsg, v_nsg, Hubbard_l, Hubbard_lmax, apply_U, orbital_resolved 
   USE xc_lib,           ONLY : xclib_dft_is
   USE scf,              ONLY : scf_type
   USE cell_base,        ONLY : alat
@@ -29,6 +29,11 @@ SUBROUTINE v_of_rho( rho, rho_core, rhog_core, &
   USE tsvdw_module,     ONLY : tsvdw_calculate, UtsvdW
   USE libmbd_interface, ONLY : mbd_interface
   USE sic_mod,          ONLY : add_vsic
+#if defined (__OSCDFT)
+  USE plugin_flags,     ONLY : use_oscdft
+  USE oscdft_base,      ONLY : oscdft_ctx
+  USE oscdft_functions, ONLY : oscdft_v_constraint
+#endif  
   !
   IMPLICIT NONE
   !
@@ -64,6 +69,9 @@ SUBROUTINE v_of_rho( rho, rho_core, rhog_core, &
   !
   ! ... calculate exchange-correlation potential
   !
+  !$acc data copyin(rho,v)
+  !$acc data copyin(rho%of_r,rho%of_g,rho_core,rhog_core) copyout(v%of_r,v%kin_r)
+  !
   IF (xclib_dft_is('meta')) THEN
      CALL v_xc_meta( rho, rho_core, rhog_core, etxc, vtxc, v%of_r, v%kin_r )
   ELSE
@@ -78,6 +86,9 @@ SUBROUTINE v_of_rho( rho, rho_core, rhog_core, &
   !
   CALL v_h( rho%of_g(:,1), ehart, charge, v%of_r )
   !
+  !$acc end data
+  !$acc end data
+  !
   ! ... DFT+U(+V): build up (extended) Hubbard potential 
   !
   IF (lda_plus_u) THEN
@@ -86,7 +97,19 @@ SUBROUTINE v_of_rho( rho, rho_core, rhog_core, &
         !
         ! DFT+U (simplified)
         !
-        CALL v_hubbard (rho%ns, v%ns, eth)
+        IF (noncolin) THEN
+           IF (orbital_resolved) THEN        
+              CALL v_hubbard_resolved_nc (rho%ns_nc, v%ns_nc, eth)
+           ELSE
+              CALL v_hubbard_nc (rho%ns_nc, v%ns_nc, eth)
+           ENDIF
+        ELSE
+           IF (orbital_resolved) THEN
+              CALL v_hubbard_resolved(rho%ns, v%ns, eth)
+           ELSE
+              CALL v_hubbard (rho%ns, v%ns, eth)
+           ENDIF
+        ENDIF   
         !
         ! Background
         IF (ldmx_b.GT.0) THEN
@@ -108,8 +131,11 @@ SUBROUTINE v_of_rho( rho, rho_core, rhog_core, &
         !
         ! DFT+U+V (simplified)
         !
-        CALL v_hubbard_extended (nsg, v_nsg, eth)
-        !
+        IF (noncolin) THEN
+           CALL v_hubbard_extended_nc (nsg, v_nsg, eth)
+        ELSE
+           CALL v_hubbard_extended (nsg, v_nsg, eth)
+        ENDIF
      ELSE
         !
         CALL errore('v_of_rho', 'Not allowed value of lda_plus_u_kind',1)
@@ -118,6 +144,18 @@ SUBROUTINE v_of_rho( rho, rho_core, rhog_core, &
      !
   ENDIF
   !
+#if defined (__OSCDFT)
+  IF (use_oscdft .AND. (oscdft_ctx%inp%oscdft_type==2)) THEN
+     IF (lda_plus_u) THEN
+        IF (lda_plus_u_kind == 0) THEN
+           CALL oscdft_v_constraint (oscdft_ctx, Hubbard_lmax, Hubbard_l, rho%ns, v%ns, eth)
+        ELSEIF (lda_plus_u_kind == 2) THEN
+           CALL oscdft_v_constraint_extended (nsg, v_nsg, eth)
+        ENDIF
+     ENDIF
+  ENDIF
+#endif
+  !
   ! ... add an electric field
   ! 
   DO is = 1, nspin_lsda
@@ -568,15 +606,23 @@ SUBROUTINE v_xc( rho, rho_core, rhog_core, etxc, vtxc, v )
   !
   CALL gradcorr( rho%of_r, rho%of_g, rho_core, rhog_core, etxc, vtxc, v )
   !
-  !$acc end data
-  !$acc end data
-  !
   ! ... to avoid NaN in some rare cases (see summations in subroutine delta_e)
-  IF ( nspin==4 .AND. .NOT.domag ) v(:,2:nspin) = 0.D0
+  IF ( nspin==4 .AND. .NOT.domag ) THEN
+     !$acc kernels
+     v(:,2:nspin) = 0.D0
+     !$acc end kernels
+  ENDIF
   !
   ! ... add non local corrections (if any)
   !
-  IF ( dft_is_nonlocc() ) CALL nlc( rho%of_r, rho_core, nspin, etxc, vtxc, v )
+  IF ( dft_is_nonlocc() ) THEN
+    !$acc update self(v)
+    CALL nlc( rho%of_r, rho_core, nspin, etxc, vtxc, v )
+    !$acc update device(v)
+  ENDIF
+  !
+  !$acc end data
+  !$acc end data
   !
   CALL mp_sum(  vtxc , intra_bgrp_comm )
   CALL mp_sum(  etxc , intra_bgrp_comm )
@@ -627,7 +673,11 @@ SUBROUTINE v_h( rhog, ehart, charge, v )
   !
   CALL start_clock( 'v_h' )
   !
+  !$acc data copyin(rhog) copy(v)
+  !
   ALLOCATE( aux(dfftp%nnr), aux1(2,ngm), vh(dfftp%nnr) )
+  !$acc data create(aux,aux1,vh)
+  !
   charge = 0.D0
   !
   IF ( gstart == 2 ) THEN
@@ -645,16 +695,23 @@ SUBROUTINE v_h( rhog, ehart, charge, v )
      ! ... calculate modified Hartree potential for ESM
      !
      CALL esm_hartree( rhog, ehart, aux )
+     !$acc update device(aux)
      !
   ELSE
      !
-     ehart     = 0.D0
+     ehart = 0.D0
+     !$acc kernels
      aux1(:,:) = 0.D0
+     !$acc end kernels
      !
      IF (do_cutoff_2D) THEN  !TS
         CALL cutoff_hartree(rhog(:), aux1, ehart)
      ELSE
-!$omp parallel do private( fac, rgtot_re, rgtot_im ), reduction(+:ehart)
+#if defined(_OPENACC)
+        !$acc parallel loop reduction(+:ehart)
+#elif defined(__OPENMP)
+        !$omp parallel do private( fac, rgtot_re, rgtot_im ), reduction(+:ehart)
+#endif
         DO ig = gstart, ngm
            !
            fac = 1.D0 / gg(ig) 
@@ -668,14 +725,18 @@ SUBROUTINE v_h( rhog, ehart, charge, v )
            aux1(2,ig) = rgtot_im * fac
            !
         ENDDO
-!$omp end parallel do
+#if defined(__OPENMP)
+        !$omp end parallel do
+#endif
      ENDIF
      !
      fac = e2 * fpi / tpiba2
      !
      ehart = ehart * fac
      !
-     aux1 = aux1 * fac
+     !$acc kernels
+     aux1(:,:) = aux1(:,:) * fac
+     !$acc end kernels
      !
      IF ( gamma_only ) THEN
         !
@@ -689,17 +750,25 @@ SUBROUTINE v_h( rhog, ehart, charge, v )
      !
      IF (do_comp_mt) THEN
         ALLOCATE( vaux(ngm), rgtot(ngm) )
+        !$acc data create(vaux,rgtot)
+        !$acc kernels
         rgtot(:) = rhog(:)
+        !$acc end kernels
         CALL wg_corr_h( omega, ngm, rgtot, vaux, eh_corr )
+        !$acc kernels
         aux1(1,1:ngm) = aux1(1,1:ngm) + REAL( vaux(1:ngm))
         aux1(2,1:ngm) = aux1(2,1:ngm) + AIMAG(vaux(1:ngm))
+        !$acc end kernels
         ehart = ehart + eh_corr
+        !$acc end data
         DEALLOCATE( rgtot, vaux )
      ENDIF
      !
      CALL mp_sum( ehart, intra_bgrp_comm )
      !
+     !$acc kernels
      aux(1:ngm) = CMPLX( aux1(1,1:ngm), aux1(2,1:ngm), KIND=DP )
+     !$acc end kernels
      !
   ENDIF
   !
@@ -711,20 +780,27 @@ SUBROUTINE v_h( rhog, ehart, charge, v )
   !
   IF ( nspin == 4 ) THEN
      !
+     !$acc kernels
      v(:,1) = v(:,1) + vh(:)
+     !$acc end kernels
      !
   ELSE
      !
      DO is = 1, nspin
         !
+        !$acc kernels
         v(:,is) = v(:,is) + vh(:)
+        !$acc end kernels
         !
      ENDDO
      !
   ENDIF
   !
+  !$acc end data
   DEALLOCATE( aux, aux1, vh )
   !
+  !$acc end data
+  !
   CALL stop_clock( 'v_h' )
   !
   RETURN
@@ -785,6 +861,7 @@ SUBROUTINE v_hubbard( ns, v_hub, eth )
   !! Hubbard potential
   REAL(DP), INTENT(OUT) :: eth
   !! Hubbard energy
+  COMPLEX(DP) :: check
   !
   !  ... local variables
   !
@@ -796,6 +873,7 @@ SUBROUTINE v_hubbard( ns, v_hub, eth )
   sgn(2) = -1.d0
   !
   v_hub(:,:,:,:) = 0.d0
+  check = (0.0,0.0)
   !
   DO na = 1, nat
      !
@@ -814,10 +892,12 @@ SUBROUTINE v_hubbard( ns, v_hub, eth )
               eth = eth + ( Hubbard_alpha(nt) + 0.5D0*effU )*ns(m1,m1,is,na)
               v_hub(m1,m1,is,na) = v_hub(m1,m1,is,na) + &
                                    Hubbard_alpha(nt)  + 0.5D0*effU
+               check = check + Hubbard_alpha(nt)  + 0.5D0*effU
               DO m2 = 1, 2 * Hubbard_l(nt) + 1
                  eth = eth - 0.5D0 * effU * ns(m2,m1,is,na)* ns(m1,m2,is,na)
                  v_hub(m1,m2,is,na) = v_hub(m1,m2,is,na) - &
                                         effU * ns(m2,m1,is,na)
+                 check = check -effU * ns(m2,m1,is,na)
               ENDDO
            ENDDO
         ENDDO
@@ -850,6 +930,7 @@ SUBROUTINE v_hubbard( ns, v_hub, eth )
   !
   IF ( iverbosity > 0 .AND. .NOT.dfpt_hub ) THEN
      WRITE(stdout,'(/5x,"HUBBARD ENERGY = ",f9.4,1x," (Ry)")') eth
+     !write(stdout,*) "check coll U", check
   ENDIF
   !
   RETURN
@@ -857,6 +938,106 @@ SUBROUTINE v_hubbard( ns, v_hub, eth )
 END SUBROUTINE v_hubbard
 !-----------------------------------------------------------------------
 
+!----------------------------------------------------------------------
+SUBROUTINE v_hubbard_nc( ns, v_hub, eth )
+  !---------------------------------------------------------------------
+  !
+  !! Computes Hubbard potential and Hubbard energy in the \textbf{noncollinear formulation}.
+  !! DFT+U: Simplified rotationally-invariant formulation by
+  !! Dudarev et al., Phys. Rev. B 57, 1505 (1998).
+  !! DFT+U+J0: B. Himmetoglu et al., Phys. Rev. B 84, 115108 (2011).
+  !
+  USE kinds,                ONLY : DP
+  USE ions_base,            ONLY : nat, ityp
+  USE ldaU,                 ONLY : Hubbard_lmax, Hubbard_l, Hubbard_U, &
+                                   Hubbard_alpha, Hubbard_J0, Hubbard_beta
+  USE lsda_mod,             ONLY : nspin
+  USE control_flags,        ONLY : iverbosity, dfpt_hub
+  USE io_global,            ONLY : stdout
+  !
+  IMPLICIT NONE
+  !
+  COMPLEX(DP) :: ns(2*Hubbard_lmax+1,2*Hubbard_lmax+1,nspin,nat)
+  !! Occupation matrix
+  COMPLEX(DP) :: v_hub(2*Hubbard_lmax+1,2*Hubbard_lmax+1,nspin,nat)
+  !! Hubbard potential
+  REAL(DP), INTENT(OUT) :: eth
+  !! Hubbard energy
+  !
+  !  ... local variables
+  !
+  INTEGER  :: is, is1, na, nt, m1, m2
+  !
+  eth    = 0.d0
+  v_hub(:,:,:,:) = 0.d0
+  !
+  DO na = 1, nat
+     !
+     nt = ityp (na)
+     !
+     IF (Hubbard_U(nt) /= 0.d0) THEN
+        DO is = 1, nspin
+           !
+           IF (is == 2) THEN
+            is1 = 3
+           ELSEIF (is == 3) THEN
+            is1 = 2
+           ELSE
+            is1 = is
+           ENDIF
+           !
+           ! Non spin-flip contribution
+           ! (diagonal [spin indexes] occupancy matrices)
+           IF (is1 == is) THEN
+              !     
+              ! diagonal part [spin indexes]     
+              DO m1 = 1, 2*Hubbard_l(nt) + 1
+                 ! Hubbard energy
+                 eth = eth + ( Hubbard_alpha(nt) + 0.5D0*Hubbard_U(nt) )&
+                             * ns(m1,m1,is,na)
+                 ! Hubbard potential
+                 v_hub(m1,m1,is,na) = v_hub(m1,m1,is,na) + & 
+                                      Hubbard_alpha(nt) + 0.5D0*Hubbard_U(nt)
+                 ! 
+                 ! NON-diagonal part [spin indexes]
+                 DO m2 = 1, 2 * Hubbard_l(nt) + 1
+                    ! Hubbard energy
+                    eth = eth - 0.5D0 * Hubbard_U(nt) * ns(m1,m2,is,na)*ns(m2,m1,is,na)
+                    ! Hubbard potential
+                    v_hub(m1,m2,is,na) = v_hub(m1,m2,is,na) - &
+                                         Hubbard_U(nt) * ns(m2,m1,is,na)
+                 ENDDO
+              ENDDO  
+           !
+           ! Spin-flip contribution
+           ! (NON-diagonal [spin indexes] occupancy matrices)   
+           ELSE
+              DO m1 = 1, 2*Hubbard_l(nt) + 1
+                 DO m2 = 1, 2 * Hubbard_l(nt) + 1 
+                    ! Hubbard energy
+                    eth = eth - 0.5D0 * Hubbard_U(nt) &
+                          * ns(m1,m2,is,na)*ns(m2,m1,is1,na)
+                    ! Hubbard potential
+                    v_hub(m1,m2,is,na) = v_hub(m1,m2,is,na) - &
+                                         Hubbard_U(nt) * ns(m2,m1,is1,na)
+                 ENDDO
+              ENDDO         
+           ENDIF
+        ENDDO  
+     ENDIF
+     !
+  ENDDO
+  !
+  ! Hubbard energy
+  !
+  IF ( iverbosity > 0 ) THEN
+     WRITE(stdout,'(/5x,"HUBBARD ENERGY = ",f9.4,1x," (Ry)")') eth
+  ENDIF
+  !
+  RETURN
+  !
+END SUBROUTINE v_hubbard_nc
+!---------------------------------------------------------------------------
 !---------------------------------------------------------------------------
 SUBROUTINE v_hubbard_b (ns, v_hub, eth)
   !-------------------------------------------------------------------------
@@ -1293,10 +1474,13 @@ SUBROUTINE v_hubbard_extended (nsg, v_hub, eth)
   ! Local variables 
   !
   INTEGER :: is, isop, na, na1, na2, nt, nt1, nt2, m1, m2, viz, equiv_na2, i_type
+  COMPLEX(DP) :: check, check_en
   INTEGER, EXTERNAL :: type_interaction, find_viz
   !
   eth  = 0.d0
   v_hub(:,:,:,:,:) = (0.d0, 0.d0)
+  check = (0.d0, 0.d0)
+  check_en = (0.d0, 0.d0)
   !
   DO na1 = 1, nat
      !
@@ -1327,9 +1511,13 @@ SUBROUTINE v_hubbard_extended (nsg, v_hub, eth)
                         !
                         v_hub(m2,m1,viz,na1,is) = &
                            - CONJG(nsg(m2,m1,viz,na1,is)) * Hubbard_V(na1,na2,i_type)
+                        check = check  - CONJG(nsg(m2,m1,viz,na1,is)) * Hubbard_V(na1,na2,i_type)
+
                         !
                         eth = eth - nsg(m2,m1,viz,na1,is) * CONJG(nsg(m2,m1,viz,na1,is)) &
                                     * Hubbard_V(na1,na2,i_type) * 0.5d0
+                        check_en = check_en -nsg(m2,m1,viz,na1,is) * CONJG(nsg(m2,m1,viz,na1,is)) &
+                                    * Hubbard_V(na1,na2,i_type) * 0.5d0
                         !
                      ENDDO
                   ENDDO
@@ -1346,9 +1534,12 @@ SUBROUTINE v_hubbard_extended (nsg, v_hub, eth)
                         !
                         v_hub(m1,m1,na,na1,is) = v_hub(m1,m1,na,na1,is) &
                                        + Hubbard_V(na1,na1,i_type) * 0.5d0
+                        check = check  + Hubbard_V(na1,na1,i_type) * 0.5d0
                         ! 
                         eth = eth + nsg(m1,m1,na,na1,is) &
                                        * Hubbard_V(na1,na1,i_type) * 0.5d0
+                        check_en = check_en +nsg(m1,m1,na,na1,is) &
+                                       * Hubbard_V(na1,na1,i_type) * 0.5d0
                         !
                      ENDDO
                      !
@@ -1437,6 +1628,8 @@ SUBROUTINE v_hubbard_extended (nsg, v_hub, eth)
   !
   IF ( iverbosity > 0 .AND. .NOT.dfpt_hub ) THEN
      WRITE(stdout,'(/5x,"HUBBARD ENERGY = ",f9.4,1x," (Ry)")') eth
+     !write(stdout,*) "check col UV",  check
+     !write(stdout,*) "check_en col UV",  check_en
   ENDIF
   !
   RETURN
@@ -1444,6 +1637,490 @@ SUBROUTINE v_hubbard_extended (nsg, v_hub, eth)
 END SUBROUTINE v_hubbard_extended
 !---------------------------------------------------------------------
 
+SUBROUTINE v_hubbard_extended_nc (nsg, v_hub, eth)
+   !-----------------------------------------------------------------------------------
+   !
+   !! Computes extended Hubbard potential and Hubbard energy.
+   !! DFT+U+V: Simplified rotationally-invariant formulation by
+   !! V.L. Campo Jr and M. Cococcioni, J. Phys.: Condens. Matter 22, 055602 (2010).
+   !
+   USE kinds,             ONLY : DP
+   USE ions_base,         ONLY : nat, ityp
+   USE ldaU,              ONLY : Hubbard_l, Hubbard_alpha, Hubbard_J0, Hubbard_beta,   &
+                                 ldim_u, ldmx_tot, max_num_neighbors, at_sc, neighood, &
+                                 Hubbard_V, Hubbard_alpha_back, is_hubbard, is_hubbard_back
+   USE lsda_mod,          ONLY : nspin
+   USE control_flags,     ONLY : iverbosity, dfpt_hub
+   USE io_global,         ONLY : stdout
+   USE noncollin_module,  ONLY : npol
+   !
+   IMPLICIT NONE
+   !
+   COMPLEX(DP), INTENT(IN)  :: nsg  (ldmx_tot, ldmx_tot, max_num_neighbors, nat, nspin)
+   COMPLEX(DP), INTENT(OUT) :: v_hub(ldmx_tot, ldmx_tot, max_num_neighbors, nat, nspin)
+   REAL(DP),    INTENT(OUT) :: eth
+   COMPLEX(DP) :: check, check_en
+   ! 
+   ! Local variables 
+   !
+   INTEGER :: is, is1, isop, na, na1, na2, nt, nt1, nt2, m1, m2, viz, equiv_na2
+   INTEGER, EXTERNAL :: find_viz
+   !
+   eth  = 0.d0
+   v_hub(:,:,:,:,:) = (0.d0, 0.d0)
+   check = (0.0,0.0)
+   check_en = (0.0,0.0)
+   !
+   !write(stdout,*) nsg
+   DO na1 = 1, nat
+      !
+      nt1 = ityp(na1)
+      !
+      IF ( is_hubbard(nt1) ) THEN
+         !
+         DO is = 1, nspin
+            !
+            IF (is == 2) THEN
+               is1 = 3
+            ELSEIF (is == 3) THEN
+               is1 = 2
+            ELSE
+               is1 = is
+            ENDIF
+            DO viz = 1, neighood(na1)%num_neigh
+               !
+               na2 = neighood(na1)%neigh(viz)
+               equiv_na2 = at_sc(na2)%at
+               nt2 = ityp(equiv_na2)
+               !
+               IF (is_hubbard(nt2) .AND. &
+                   (Hubbard_V(na1,na2,1).NE.0.d0) ) THEN
+                   !
+                   ! Here no need to use is1: complex conjugation is enough
+                   ! For both standard and background states of a center atom
+                   DO m1 = 1, ldim_u(nt1)
+                      ! For both standard and background states of the neighbor atom
+                      DO m2 = 1, ldim_u(nt2)
+                         !
+                         v_hub(m2,m1,viz,na1,is) = - CONJG(nsg(m2,m1,viz,na1,is)) * Hubbard_V(na1,na2,1)
+                        check = check - CONJG(nsg(m2,m1,viz,na1,is)) * Hubbard_V(na1,na2,1)
+                         !
+                         eth = eth - nsg(m2,m1,viz,na1,is) * CONJG(nsg(m2,m1,viz,na1,is)) &
+                                     * Hubbard_V(na1,na2,1) * 0.5d0
+                        check_en = check_en - nsg(m2,m1,viz,na1,is) * CONJG(nsg(m2,m1,viz,na1,is)) &
+                                     * Hubbard_V(na1,na2,1) * 0.5d0
+                         !
+                      ENDDO
+                   ENDDO
+                   !
+                   IF ( na1.EQ.na2 .AND. is1.EQ.is) THEN
+                      !
+                      na = find_viz(na1,na1)
+                      !
+                      ! This is the diagonal term (like in the DFT+U only case)
+                      ! 
+                      DO m1 = 1, ldim_u(nt1)
+                         !
+                         v_hub(m1,m1,na,na1,is) = v_hub(m1,m1,na,na1,is) &
+                                        + Hubbard_V(na1,na1,1) * 0.5d0
+                        check = check + Hubbard_V(na1,na1,1) * 0.5d0
+                         ! 
+                         eth = eth + nsg(m1,m1,na,na1,is) &
+                                        * Hubbard_V(na1,na1,1) * 0.5d0
+                        check_en = check_en + nsg(m1,m1,na,na1,is) &
+                                        * Hubbard_V(na1,na1,1) * 0.5d0
+                         !
+                      ENDDO
+                      !
+                   ENDIF
+                   !
+               ENDIF
+               !
+            ENDDO ! viz
+            !
+         ENDDO ! is
+         !  
+      ENDIF
+      !
+      ! Hubbard_alpha
+      !!
+      IF ( ldim_u(nt1).GT.0 .AND. (Hubbard_alpha(nt1).NE.0.0d0 ) ) THEN
+         !
+         na = find_viz(na1,na1)
+         !
+         DO is = 1, npol
+            !
+            DO m1 = 1, 2*Hubbard_l(nt1)+1
+               v_hub(m1,m1,na,na1,is**2) = v_hub(m1,m1,na,na1,is**2) + Hubbard_alpha(nt1)
+               eth = eth + nsg(m1,m1,na,na1,is**2)* Hubbard_alpha(nt1)
+            ENDDO
+            !
+         ENDDO
+         !
+      ENDIF
+      !
+   ENDDO ! na1
+   !
+   !
+   IF (nspin.EQ.1) eth = eth * 2.d0
+   !
+   ! Hubbard energy
+   !
+   IF ( iverbosity > 0 .AND. .NOT.dfpt_hub ) THEN
+      WRITE(stdout,'(/5x,"HUBBARD ENERGY = ",f9.4,1x," (Ry)")') eth
+   ENDIF
+   !
+   RETURN
+   !
+ END SUBROUTINE v_hubbard_extended_nc
+!
+!-----------------------------------------------------------------------
+SUBROUTINE v_hubbard_resolved( ns, v_hub, eth )
+!---------------------------------------------------------------------
+!
+!! Computes Hubbard potential and Hubbard energy
+!! for a manifold of selected spin- or magnetic quantum orbitals.
+!! The Hubbard potential is first calculated in the diagonal representation
+!! based on the eigenvalues of the occupation matrix and then 
+!! re-rotated using the eigenvectors in order to retain compatiblity with
+!! other parts of the code.
+!! See Macke et al., arXiv:2312.13580 (2023).
+!! Uses the simplified rotationally-invariant formulation by
+!! Dudarev et al., Phys. Rev. B 57, 1505 (1998).
+!
+USE kinds,                ONLY : DP
+USE ions_base,            ONLY : nat, ityp
+USE ldaU,                 ONLY : Hubbard_lmax, Hubbard_l, Hubbard_Um, &
+                                 Hubbard_alpha_m, lambda_ns, &
+                                 eigenvecs_ref, order_um, apply_U, hub_pot_fix
+USE lsda_mod,             ONLY : nspin
+USE constants,            ONLY : eps16, RYTOEV
+USE control_flags,        ONLY : iverbosity, dfpt_hub
+USE io_global,            ONLY : stdout
+!
+IMPLICIT NONE
+!
+REAL(DP), INTENT(IN)  :: ns(2*Hubbard_lmax+1,2*Hubbard_lmax+1,nspin,nat)
+!! occupation matrix
+REAL(DP), INTENT(OUT) :: v_hub(2*Hubbard_lmax+1,2*Hubbard_lmax+1,nspin,nat)
+!! Hubbard potential
+REAL(DP), INTENT(OUT) :: eth
+!! Hubbard energy
+!
+!  ... local variables
+!
+! Hubbard potential in the diagonal representation
+REAL(DP)                 :: v_hub_diag(2*Hubbard_lmax+1,nspin,nat), temp
+REAL(DP)                 :: effU, effalpha
+! eigenvectors of the ns occupation matrix in the current iteration
+COMPLEX(DP)              :: eigenvecs_current(2*Hubbard_lmax+1,2*Hubbard_lmax+1,nspin)
+!
+INTEGER                  :: is, na, nt, m1, m2, m3, ldim, m_order
+! the ordering vector for the orbital-tracking routine
+INTEGER                  :: order(2*Hubbard_lmax+1)
+LOGICAL                  :: is_first
+IF (.NOT. ALLOCATED(order_um)) THEN 
+  IF (nspin == 2 ) THEN 
+     ALLOCATE(order_um(2*Hubbard_lmax+1,nspin, nat))  
+  ELSE 
+     ALLOCATE(order_um(2*Hubbard_lmax+1,1,nat)) 
+  END IF 
+  order_um = 0
+END IF 
+
+!
+!
+eth    = 0.d0
+lambda_ns(:,:,:) = 0.d0
+! orbital occupations (=eigenvalues of rho%ns)
+!
+v_hub(:,:,:,:) = 0.d0
+v_hub_diag(:,:,:) = 0.d0 
+!
+IF ( .NOT. apply_U ) RETURN
+! Do not apply corrections before the eigenstates have stabilized
+! The eigenstates are considered stable a) when starting from a
+! converged charge density or b) when the treshold of iterative 
+! diagonalization gets small enough (typically within 3-6 iterations).
+! This is controlled by electrons.f90.
+! 
+DO na = 1, nat
+   !
+   nt = ityp (na)
+   !
+   IF ( ANY(ABS(Hubbard_Um(:,:,nt)) .GT. eps16) .OR. &
+        ANY(ABS(Hubbard_alpha_m(:,:,nt)) .GT. eps16) ) THEN       
+      !
+      ldim = 2 * Hubbard_l(nt) + 1
+      eigenvecs_current(:,:,:) = CMPLX(0.d0,0.d0, kind=dp)
+      is_first = ALL(eigenvecs_ref(:,:,:,na) == eigenvecs_current)  
+
+      !
+      effU = 0.0
+      effalpha = 0.0
+      !
+      CALL diag_ns( ldim, ns(1:ldim,1:ldim,:,na), lambda_ns(1:ldim,:,na), &
+                    eigenvecs_current(1:ldim,1:ldim,:) )
+      !
+      DO is = 1, nspin
+         !
+         ! sort eigenvectors with respect to the (reference) order established in eigvecs_first
+         IF (is_first)  THEN  
+             order(1:ldim) = order_um(1:ldim,is,na)
+             IF (ALL(order(1:ldim)==0)) order(1:ldim) = [(m1,m1=1,ldim)] 
+             DO m1 =1, ldim 
+               eigenvecs_ref(1:ldim, order(m1), is, na) = eigenvecs_current(1:ldim, m1, is) 
+             END DO 
+         END IF 
+         order(:) = 0
+         CALL order_eigenvecs( order(1:ldim), eigenvecs_current(1:ldim,1:ldim,is), &
+                                 eigenvecs_ref(1:ldim,1:ldim,is,na), ldim )
+         !
+         
+         order_um(1:ldim,is,na) = order(1:ldim) 
+         DO m1 = 1, ldim
+            !
+            ! calculate Hubbard potential and -energy
+            ! using the eigenvalues and their ordering
+            m_order = order(m1)
+            effU = Hubbard_Um(m_order,is,nt)
+            effalpha = Hubbard_alpha_m(m_order,is,nt)
+            !
+            ! linear Hubbard U terms:
+            eth = eth + ( effalpha + 0.5D0*effU ) * lambda_ns(m1,is,na)
+            v_hub_diag(m1,is,na) = v_hub_diag(m1,is,na) + &
+                                       effalpha + 0.5D0*effU
+            ! quadratic Hubbard U terms:
+            eth = eth - 0.5D0 * effU * lambda_ns(m1,is,na) * lambda_ns(m1,is,na)
+            v_hub_diag(m1,is,na) = v_hub_diag(m1,is,na) - &
+                                       effU * lambda_ns(m1,is,na)
+            !
+            ! The following can be eliminated once code is approved
+#if defined(__DEBUG)
+            WRITE( stdout,'(/5x,"v_of_rho:")')
+            WRITE( stdout,'(/5x,"AT:",i2," ,m_order:",i2,", is: ",i2,", LAMBDA:",f7.5," U:",f7.5)') &
+                  na,m_order,is,lambda_ns(m1,is,na),effU*RYTOEV
+            IF ( effalpha /= 0.d0) &
+               WRITE( stdout,'(/5x,"m_order:",i2,", is: ",i2,", LAMBDA:",f8.5," alpha:",f8.5)') &
+                  m_order,is,lambda_ns(m1,is,na),effalpha*RYTOEV
+            WRITE( stdout,'(/5x,"Hubbard potential of this state (v_hub_diag) = ",f8.5)') &
+                  v_hub_diag(m1,is,na)
+            WRITE( stdout,'(/5x,"Cumulative running total of Hubbard energies (eth) = ",f8.5)') eth
+#endif
+            !
+         ENDDO
+         !
+         ! backrotation of v_hub_diag to the non-diagonal v_hub
+         DO m1 = 1, ldim
+            DO m2 = 1, ldim
+               temp = CMPLX(0.d0,0.d0, kind=dp)
+               DO m3 = 1, ldim
+                  temp = temp + CONJG(eigenvecs_current(m1,m3,is))* &
+                     v_hub_diag(m3,is,na)*eigenvecs_current(m2,m3,is)
+               ENDDO
+               v_hub(m1,m2,is,na) = DBLE(temp)
+            ENDDO
+         ENDDO
+      ENDDO ! is
+      !
+      IF ( ALL(eigenvecs_ref(:,:,:,na) .EQ. 0.d0) ) THEN
+         !
+         ! if this routine is executed for the first time,
+         ! save the current eigenvecs as reference eigenvecs
+         eigenvecs_ref(1:ldim,1:ldim,:,na) = eigenvecs_current(1:ldim,1:ldim,:)
+      ENDIF
+      !
+   ENDIF
+   !
+ENDDO ! nt
+!
+IF (nspin==1) eth = 2.d0 * eth
+!
+! print Hubbard energy
+!
+IF ( iverbosity > 0 .AND. .NOT.dfpt_hub ) THEN
+   WRITE(stdout,'(/5x,"HUBBARD ENERGY = ",f9.5,1x," (Ry)")') eth
+ENDIF
+!
+RETURN
+!
+END SUBROUTINE v_hubbard_resolved
+!----------------------------------------------------------------------------
+SUBROUTINE v_hubbard_resolved_nc( ns, v_hub, eth )
+!----------------------------------------------------------------------------
+!
+!! Computes Hubbard potential and Hubbard energy
+!! for a manifold of selected orbitals (noncollinear formulation).
+!! The Hubbard potential is first calculated in the diagonal representation
+!! based on the eigenvalues of the occupation matrix and then 
+!! re-rotated using the eigenvectors in order to retain compatiblity with
+!! other parts of the code.
+!! Uses the simplified rotationally-invariant formulation by
+!! Dudarev et al., Phys. Rev. B 57, 1505 (1998).
+!
+USE kinds,                ONLY : DP
+USE ions_base,            ONLY : nat, ityp
+USE ldaU,                 ONLY : Hubbard_lmax, Hubbard_l, Hubbard_Um_nc, &
+                                 Hubbard_alpha_m_nc, lambda_ns, order_um,&
+                                 eigenvecs_ref, apply_U, hub_pot_fix
+USE lsda_mod,             ONLY : nspin
+USE constants,            ONLY : eps16, RYTOEV
+USE control_flags,        ONLY : iverbosity, dfpt_hub
+USE io_global,            ONLY : stdout
+!
+IMPLICIT NONE
+!
+COMPLEX(DP), INTENT(IN) :: ns(2*Hubbard_lmax+1,2*Hubbard_lmax+1,nspin,nat)
+!! occupation matrix
+COMPLEX(DP), INTENT(OUT):: v_hub(2*Hubbard_lmax+1,2*Hubbard_lmax+1,nspin,nat)
+!! Hubbard potential
+REAL(DP), INTENT(OUT) :: eth
+!! Hubbard energy
+!
+!  ... local variables
+!
+REAL(DP)                 :: effU, effalpha
+COMPLEX(DP)              :: v_hub_diag(4*Hubbard_lmax+2,nat), temp
+COMPLEX(DP)              :: v_hub_temp(4*Hubbard_lmax+2,4*Hubbard_lmax+2)
+COMPLEX(DP)              :: eigenvecs_current(4*Hubbard_lmax+2,4*Hubbard_lmax+2)
+!
+INTEGER                  :: is, na, nt, m1, m2, m3, ldim, m_order
+!
+INTEGER                  :: order(4*Hubbard_lmax+2) 
+LOGICAL                  :: is_first
+!! ordering vector
+!
+IF (.NOT. ALLOCATED(order_um)) THEN 
+  ALLOCATE(order_um(4*Hubbard_lmax+2,1,nat)) 
+  order_um = 0
+END IF 
+
+eth    = 0.d0
+lambda_ns(:,:,:) = 0.d0
+! orbital occupations (=eigenvalues of rho%ns)
+!
+v_hub(:,:,:,:) = 0.d0
+v_hub_diag(:,:) = 0.d0 
+! Hubbard potential in the diagonal representation
+!
+IF ( .NOT. apply_U ) RETURN
+! Do not apply corrections before the eigenstates have stabilized
+! The eigenstates are considered stable a) when starting from a
+! converged charge density or b) when the treshold of iterative 
+! diagonalization gets small enough (typically within 3-6 iterations)
+! this is checked in electrons.f90
+! 
+DO na = 1, nat
+   !
+   nt = ityp (na)
+   !
+   IF ( ANY(ABS(Hubbard_Um_nc(:,nt)) .GT. eps16) .OR. &
+        ANY(ABS(Hubbard_alpha_m_nc(:,nt)) .GT. eps16) ) THEN       
+      !
+      ldim = 2 * Hubbard_l(nt) + 1
+      eigenvecs_current(:,:) = CMPLX(0.d0,0.d0, kind=dp)
+      is_first = ALL(eigenvecs_ref(:,1,:,na) == eigenvecs_current)  
+      !
+      effU = 0.0
+      effalpha = 0.0
+      !
+      CALL diag_ns_nc( ldim, ns(1:ldim,1:ldim,:,na), lambda_ns(1:2*ldim,1,na), &
+                     eigenvecs_current(1:2*ldim,1:2*ldim) )
+                              !
+      ! sort eigenvectors with respect to the (reference) order established in eigvecs_first
+      IF (is_first)  THEN  
+        order(1:2*ldim) = order_um(1:2*ldim,1,na)
+        IF (ALL(order(1:2*ldim)==0)) order(1:2*ldim) = [(m1,m1=1,2*ldim)] 
+        DO m1 =1, 2*ldim 
+          eigenvecs_ref(1:2*ldim, order(m1), 1, na) = eigenvecs_current(1:2*ldim, m1) 
+        END DO 
+      END IF 
+      order(:) = 0
+      !
+      CALL order_eigenvecs( order(1:2*ldim), eigenvecs_current(1:2*ldim,1:2*ldim), &
+                                 eigenvecs_ref(1:2*ldim,1:2*ldim,1,na), 2*ldim )
+      !
+      ! No need to iterate over is (all done in diag_ns_nc)
+      order_um(1:2*ldim,1,na) = order(1:2*ldim) 
+      DO m1 = 1, 2*ldim
+         !
+         ! calculate Hubbard potential and -energy
+         ! using the eigenvalues and their ordering
+         m_order = order(m1)
+         effU = Hubbard_Um_nc(m_order,nt)
+         effalpha = Hubbard_alpha_m_nc(m_order,nt)
+         !
+         ! linear Hubbard U terms:
+         eth = eth + ( effalpha + 0.5D0*effU ) * lambda_ns(m1,1,na)
+         v_hub_diag(m1,na) = v_hub_diag(m1,na) + &
+                                    effalpha + 0.5D0*effU
+         ! quadratic Hubbard U terms:
+         eth = eth - 0.5D0 * effU * lambda_ns(m1,1,na) * lambda_ns(m1,1,na)
+         v_hub_diag(m1,na) = v_hub_diag(m1,na) - &
+                                    effU * lambda_ns(m1,1,na)
+         !
+         ! The following block can be eliminated once code is approved
+#if defined(__DEBUG)
+         WRITE( stdout,'(/5x,"v_of_rho (NC):")')
+         WRITE( stdout,'(/5x,"AT:",i2," ,m_order:",i2,", LAMBDA:",f7.5," U:",f7.5)') &
+               na,m_order,lambda_ns(m1,1,na),effU*RYTOEV
+         IF ( effalpha /= 0.d0) &
+            WRITE( stdout,'(/5x,"m_order:",i2,", LAMBDA:",f8.5," alpha:",f8.5)') &
+               m_order,lambda_ns(m1,1,na),effalpha*RYTOEV
+         WRITE( stdout,'(/5x,"Hubbard potential of this state (v_hub_diag) = ",f8.5)') &
+               v_hub_diag(m1,na)
+         WRITE( stdout,'(/5x,"Cumulative running total of Hubbard energies (eth) = ",f8.5)') eth
+#endif
+         !
+      ENDDO
+         !
+      ! backrotation of v_hub_diag to the non-diagonal v_hub
+      ! first, go back to the 2ldim*2ldim matrix
+      v_hub_temp(:,:) = 0.d0
+      !
+      DO m1 = 1, 2*ldim
+         DO m2 = 1, 2*ldim
+            temp = CMPLX(0.d0,0.d0, kind=dp)
+            DO m3 = 1, 2*ldim
+               temp = temp + CONJG(eigenvecs_current(m1,m3))* &
+                  v_hub_diag(m3,na)*eigenvecs_current(m2,m3)
+            ENDDO
+            v_hub_temp(m1,m2) = temp
+         ENDDO
+      ENDDO
+      ! now, sort the different quadrants of v_hub_temp into the actual
+      ! v_hub. upper left quadrant is spin 1, upper right spin 2 etc.
+      !
+      DO m1 = 1, ldim
+         DO m2 = 1,ldim
+            v_hub(m1,m2,1,na) = v_hub_temp(m1,m2)
+            v_hub(m1,m2,2,na) = v_hub_temp(m1,ldim+m2)
+            v_hub(m1,m2,3,na) = v_hub_temp(ldim+m1,m2)
+            v_hub(m1,m2,4,na) = v_hub_temp(ldim+m1,ldim+m2)
+         ENDDO
+      ENDDO
+      !
+      IF ( ALL(eigenvecs_ref(:,:,1,na) .EQ. 0.d0) ) THEN
+         ! if this routine is executed for the first time,
+         ! save the current eigenvecs as reference eigenvecs
+         eigenvecs_ref(1:2*ldim,1:2*ldim,1,na) = &
+                     & eigenvecs_current(1:2*ldim,1:2*ldim)
+      ENDIF
+      !
+   ENDIF
+   !
+ENDDO ! nt
+!
+! print Hubbard energy
+!
+IF ( iverbosity > 0 .AND. .NOT.dfpt_hub ) THEN
+   WRITE(stdout,'(/5x,"HUBBARD ENERGY = ",f9.5,1x," (Ry)")') eth
+ENDIF
+!
+RETURN
+!
+END SUBROUTINE v_hubbard_resolved_nc
+!----------------------------------------------------------------------------
 !----------------------------------------------------------------------------
 SUBROUTINE v_h_of_rho_r( rhor, ehart, charge, v )
   !----------------------------------------------------------------------------
