@@ -10,11 +10,10 @@ MODULE dft_setting_routines
   !--------------------------------------------------------------------------
   !! Routines to set and/or recover DFT names, parameters and flags.
   !
-  USE xclib_utils_and_para,  ONLY: stdout
 #if defined(__LIBXC)
 #include "xc_version.h"
-  USE xc_f03_lib_m
-#endif  
+#endif
+  USE xclib_utils_and_para,  ONLY: stdout
   !
   SAVE
   !
@@ -34,6 +33,9 @@ MODULE dft_setting_routines
             xclib_finalize_libxc
   PUBLIC :: set_libxc_ext_param, get_libxc_ext_param
   PUBLIC :: capital
+#if (__LIBXC && XC_MAJOR_VERSION < 5)
+  INTEGER,PARAMETER :: XC_FAMILY_HYB_LDA=-1, XC_FAMILY_HYB_GGA=-1, XC_FAMILY_HYB_MGGA=-1
+#endif 
   !
 CONTAINS
   !
@@ -330,6 +332,7 @@ CONTAINS
                                     is_libxc, exx_fraction
 #if defined(__LIBXC)
     USE dft_setting_params,   ONLY: xc_kind_error
+    USE xc_f03_lib_m
 #endif
     USE xclib_utils_and_para, ONLY: nowarning
     ! 
@@ -496,7 +499,10 @@ CONTAINS
                                    islda, isgradient, ismeta, exx_fraction, &
                                    screening_parameter, gau_parameter,      &
                                    ishybrid, has_finite_size_correction,    &
-                                   is_libxc, exx_term
+                                   is_libxc, exx_term, max_flags
+#if defined(__LIBXC)
+    USE xc_f03_lib_m
+#endif
     !
     IMPLICIT NONE
     !
@@ -506,7 +512,7 @@ CONTAINS
 #if defined(__LIBXC)
     TYPE(xc_f03_func_t) :: xc_func
     TYPE(xc_f03_func_info_t) :: xc_info
-    INTEGER :: fkind, iid, family, id_vec(6), iflag, flags_tot, libxc_flag(16)
+    INTEGER :: fkind, iid, family, id_vec(6), iflag, flags_tot, libxc_flag(max_flags+1)
     REAL(DP) :: omega, alpha, beta
 #endif
     LOGICAL :: is_libxc13, is_libxc12
@@ -574,7 +580,8 @@ CONTAINS
         fkind = xc_f03_func_info_get_kind( xc_info )
         family = xc_f03_func_info_get_family( xc_info )
         flags_tot = xc_f03_func_info_get_flags( xc_info )
-        DO iflag = 15, 0, -1
+        !
+        DO iflag = max_flags, 0, -1
           libxc_flag(iflag+1) = 0
           IF ( flags_tot-2**iflag < 0 ) CYCLE
           libxc_flag(iflag+1) = 1
@@ -1070,12 +1077,13 @@ CONTAINS
     !! Initialize Libxc functionals, if present.
     USE dft_setting_params,  ONLY: iexch, icorr, igcx, igcc, imeta, imetac,   &
                                    is_libxc, libxc_initialized, exx_fraction, &
-                                   screening_parameter
+                                   screening_parameter, max_flags
 #if defined(__LIBXC)
     USE xclib_utils_and_para,ONLY: nowarning
     USE dft_setting_params,  ONLY: n_ext_params, xc_func, xc_info, par_list, &
                                    libxc_flags, n_ext_params, exx_term, &
                                    lxc_exx_desc, lxc_scr_desc
+    USE xc_f03_lib_m
 #endif
     IMPLICIT NONE
     INTEGER, INTENT(IN) :: xclib_nspin
@@ -1110,13 +1118,18 @@ CONTAINS
         family = xc_f03_func_info_get_family( xc_info(iid) )
         !
         flags_tot = xc_f03_func_info_get_flags( xc_info(iid) )
-        DO iflag = 15, 0, -1
+        !
+        DO iflag = max_flags, 0, -1
           libxc_flags(iid,iflag) = 0
           IF ( flags_tot-2**iflag < 0 ) CYCLE
           libxc_flags(iid,iflag) = 1
           flags_tot = flags_tot-2**iflag
         ENDDO
         !
+#if (XC_MAJOR_VERSION >= 7)
+        IF (libxc_flags(iid,17)==1) CALL xc_f03_func_set_fhc_enforcement(xc_func(iid),.FALSE.)
+#endif
+        !
         n_ext_params(iid) = xc_f03_func_info_get_n_ext_params( xc_info(iid) )
         DO ip = 0, n_ext_params(iid)-1
           par_list(iid,ip+1) = xc_f03_func_info_get_ext_params_default_value( &
@@ -1165,7 +1178,7 @@ CONTAINS
                           &/5X,"long range exx is not available yet (if needed).")' )&
                           id_vec(iid)
           IF ( libxc_flags(iid,14) == 1 ) &
-            WRITE(stdout,'(4X,"[w02] libxc functional with ID ",I4," is still ", &
+            WRITE(stdout,'(4X,"WARNING: libxc functional with ID ",I4," is still ", &
                       &/4X,"in development.")' ) id_vec(iid)
           IF ( libxc_flags(iid,15) == 1 ) &
             WRITE(stdout,'(/5X,"WARNING: libxc functional with ID ",I4," depends on",  &
@@ -1183,6 +1196,7 @@ CONTAINS
 #if defined(__LIBXC)
   SUBROUTINE xclib_init_libxc_print_version_info()
     !------------------------------------------------------------------------
+    USE xc_f03_lib_m
     IMPLICIT NONE
     INTEGER :: major, minor, micro
     CALL xc_f03_version(major, minor, micro)
@@ -1200,6 +1214,7 @@ CONTAINS
 #if defined(__LIBXC)
     USE dft_setting_params,  ONLY: xc_func, xc_kind_error, libxc_flags, &
                                    n_ext_params, par_list
+    USE xc_f03_lib_m
 #endif
     IMPLICIT NONE
     INTEGER :: iid
@@ -1235,6 +1250,7 @@ CONTAINS
     USE kind_l,       ONLY: DP
 #if defined(__LIBXC)
     USE dft_setting_params,  ONLY: xc_func, par_list
+    USE xc_f03_lib_m
 #endif
     IMPLICIT NONE
     INTEGER, INTENT(IN) :: sid
